;                    
; SEIN model: knowledge and economy levels
; Nigel Gilbert, Petra Ahrweiler, Andreas Pyka
;
; version 2.0  25 August 2003
; version .1  1 September 2003   Bugs caused by extreme parameter values removed.  
;                                Partnerships added
; version .2  17 September 2003  Networks added
; version .3  26 October  2003   Some network bugs removed and plots added
; version .4   1 November 2003   Economy added
; version .5   8 November 2003   Bugs, converted to NetLogo 2.0
; version .7  12 November 2003   Partners cannot be from networks
; version .9  13 November 2003   (Hamburg) Network bugs removed
; version 3.0 14 November 2003   Tidy up; make environmental inputs and outputs product ranges
; version .2  27 December 2003   Changed incremental research direction setting
; version .3  27 December 2003   Network graphics
; version .6   2 January  2004   Herfindahl index, bug fixes, shorter partner search
; version .7  15 February 2004   Connectivity graph; variable start-ups; random initial capital; 
;                                increased radical research threshold; changed network display
; version .8  24 January 2006    Added degree distribution plot and UCINET export
; Notes:
;   A firm cannot be a member of more than 1 network at any one time
;   In the plots, networks are included in the count of firms (but not partnerships)

; globals preceded by ';' below are set by sliders, not by code
globals [
    step                           ; time step
    initial-capital                ; the capital that a firm starts with
;   nFirms                         ; the number of firms initially
;   initial-capital-dist           ; shape of initial capital distribution
;   nProducts                      ; the number of products that are possible
;   nInputs                        ; the maximum number of inputs that a firm can require
;   nHEIs                          ; the number of HEIs at Initialization - will remain constant throughout the Simulation
    maxPrice                       ; maximum price of any product
    maxQuality                     ; maximum quality of any product
    reward-to-trigger-start-up     ; a start-up is created when the best reward in the  
                                   ;    round is more than or equal to this
    max-ih-length                  ; maximum length of an innovation hypothesis
    nCapabilities                  ; global number of capabilities possible
;   attractiveness-threshold       ; how attractive a firm must be before it becomes a
                                   ;    partner
;   HEI-success-threshold          ; how successfully have partners of a HEI to be to enable the HEI
                                   ; to create a spin-off
    ;success-threshold              ; how successful an innovation must be before it is a 
                                   ;    success
    low-capital-threshold          ; if a firm's capital is below this, it does radical
                                   ;    research
    capital-knowledge-ratio        ; proportionality between kene length and firm's 
                                   ;    capital
    HEI-capital-factor             ; the factor for multiplication of a firm's capital to make
                                   ;    it a HEI
    incr-research-tax              ; tax paid for one step of incremental research
    radical-research-tax           ; tax paid for one step of radical research
    collaboration-tax              ; tax paid for every partnership per step
    depreciation                   ; tax paid per step by every firm
    
    largest-capital                ; updated every round: the capital of the richest form
    raw-cost                       ; the cost of a unit of raw material
    raw-materials                  ; the inputs that come from the environment are
                                   ;      numbered 0 upto (but not including) this
    final-price                    ; the selling price of a unit sold to the end user
    end-products                   ; the outputs that are bought by final consumers are
                                   ;      numbered greater than nProducts - this
    activity                       ; description of the phase of activity of the model
    ; for the network display:
    x-max                          ; the size of the graphic area
    y-max 
    diam                           ; the size factor for the displayed icon for a firm
    friction                       ; friction coeff. so icons do not move too fast
    spring-force                   ; force of spring tending to pull linked icons towards each other
    spring-length                  ; springs don't pull when link length is shorter than this
    mutual-repulsion               ; counter-vailing force to springs
    knowledge-flow                 ; containing sum of flown knowledge for one time-step
    knowledge-flow-list            ; containing nodes of flown knowledge 
    age-at-death                   ; contains the ages at death of all dead firms
    ;current_run                   ; the number of the current run
    nSpin-offs                     ; holds the number of spin-offs created in a time step
    nStart-ups                     ; holds the number of start-ups created in a time step
    spin-off-Bonus                 ; holds te Bonus a HEI gets if it creates a spin-off
    
    ]
    
breed [ firms ]
breed [ networks ]
breed [ edges ]



firms-own [
    capabilities
    abilities
    expertises
    ih                              ; the firm's innovation hypothesis 
                                    ;     (the locations of the ih kene triples )
    capital                         ; the amount of capital of the firm
    product                         ; the product produced by this firm (a number)
    quality                         ; the quality of the product
    inputs                          ; the input stuff required to make the product
    research-direction              ; direction of changing an ability 
                                    ;     (for incremental research)
    ability-to-research             ; the ability that is being changed by incremental 
                                    ;     research
    last-reward                     ; the reward received by the firm last time
    advert                          ; a list of the capabilities of my innovation 
                                    ;     hypothesis
    previous-partners               ; list of firms with which I have previously 
                                    ;     partnered
    time-in-HEI-coop                ; incremental counter of time-steps a firm is 
                                    ;     cooperating with a HEI
    time-since-HEI-coop             ; incremental counter of time steps a firm did 
                                    ;     not cooperate with a HEI                              
    partners                        ; list of my current partners
    suppliers                       ; list of suppliers
    customers                       ; list of customers
    selling?                        ; whether I have made any product to sell this round 
                                    ;     (I can make product only if all my inputs are 
                                    ;     available to me)
    total-cost                      ; the total of the prices charged by suppliers for my inputs
    price                           ; the price I want to sell the product for
    sales                           ; the total amount received from selling in this round
    nCustomers                      ; number of customers who bought my product
    network                         ; the network I am a member of
    age                             ; steps since the firm was started
    network-firm                    ; is this a firm representing a network?
    HEI                             ; is this a firm representing a HEI?
    ]
    
networks-own [
    firm                            ; the firm object that does this network's work
    members                         ; the firms that constitute this network
    ]
    
edges-own [
  a 
  b
  kind
  ]

to setup
    let oname ""
    ifelse ( current_run < 10 )
      [ set oname (word  "SKIN3_8run0" current_run "_exp.txt") ]
      [ set oname (word  "SKIN3_8run" current_run "_exp.txt") ]      
    file-open oname
    clear-all
    clear-output
    no-display
    set activity "Initialising"
    set step 0
    set initial-capital 40000
;   set nProducts 50                   ; set with an interface slider
;   set nFirms 100                     ; set with an interface slider
;   set nInputs 4                      ; set with an interface slider
;   set nHEIs 20                       ; set with an interface slider
    set maxPrice 1000
    set reward-to-trigger-start-up 400 
    set max-ih-length 9
    set nCapabilities 1000
;   set attractiveness-threshold 0.4  ; set with an interface slider
;    set success-threshold 200          
    set low-capital-threshold 1000     
    ;set capital-knowledge-ratio 50
    set capital-knowledge-ratio 2000
    set incr-research-tax 100
    set radical-research-tax 100
    set collaboration-tax 10
    set HEI-capital-factor 5
    set age-at-death []
        
    set raw-cost 1
    set final-price 1000
    ifelse open-system 
      [ set raw-materials 5 set end-products 5] 
      [ set raw-materials 0 set end-products 0 ]
    
    setup-layout
    create-firms nFirms + nHEIs            ; create a population of firms
    ask firms [ setup-firm ]
    make-ldfs
    make-HEIs
    if plotting [ repeat 5 [ layout-firms ] ]
    display
    if plotting [ show-plots ]
    set activity "Initialised"
    header-write
end

; set up globals for the network diagram


to setup-layout  
  set diam 1.5
  set x-max max-pxcor - (diam / 2) + 1
  set y-max max-pycor - (diam / 2) + 1
  set-default-shape firms "factory"
  set-default-shape edges "line"
  set friction 0.25    
  set spring-force 0.11 ; 0.2
  set spring-length 10.0 ;2.0
  set mutual-repulsion 8 ; 2.5
  ask patches [set pcolor white]
end

; set up a single firm with all its required knowledge

to setup-firm
    initialise-firm
    make-kene
    make-innovation-hypothesis
    make-advert
    manufacture                ; not strictly necessary, but sets the initial product.
end

; make a 'null' firm

to initialise-firm
    set capital random initial-capital
    set ih []
    set research-direction "random"
    set partners []
    set previous-partners []
    set suppliers []
    set customers []
    set network []
    set network-firm false
    set age 0
    set price (random maxPrice) + 1  ; price of my product is initially random
    set selling? true                ; start by assuming that all suppliers are able to supply
    set capabilities []              ; create a null kene
    set abilities []
    set expertises []
    set HEI false                    ; by initialization a firm is not a HEI (it can be made one
                                     ;   during the procedure make-HEI
    setxy random-float (x-max) * (2 * (random 2) - 1)
          random-float (y-max) * (2 * (random 2) - 1)
    set heading 0
end

; give some firms ten times the initial capital of the others (but not firms that make raw-materials)

to make-ldfs
    let ldf 0
   
    repeat n-big-firms [
        set ldf one-of firms with [ product = ((random nProducts - raw-materials) + raw-materials) and 
                                           capital = initial-capital]
        if ldf != nobody [ ask ldf [ set capital capital * 10 ] ]
    ]
end


; select nHEIs firms of the population of firms and make them HEIs (unable them to produce and
; give them more initial capital for a wider knowledge base
; (setup-procedure)

to make-HEIs
     let current-HEI one-of firms with [ HEI = false ]
  
     repeat nHEIs [
         set current-HEI one-of firms with [ HEI = false ]
         ask current-HEI [ 
             ;set capital capital * HEI-capital-factor
             set HEI true         
             set shape "arrow"
             set heading 0
             set color 0
             set product -1
             ]
         
         
         ]
end
to go
    set knowledge-flow 0
    set knowledge-flow-list []
    set activity ""
    if count firms with [ not HEI ] <= 5 or step > maxSteps[ 
      file-close      
      ifelse current_run < nRuns
        [ set step 0
          set current_run current_run + 1
          ifelse current_run >= 25 
            [ set nHEIs 0 ]
            [ set nHEIs 20 ]
        ]
        [ stop ]
      setup                                    ; stop if all firms have gone bankrupt
      ]               
    if activity = 0 [ setup ]                  ; if the setup button hasn't been pressed, 
                                               ;     do it now
    if activity = "Initialising" [ stop ]      ; if setup is still in progress, don't start
    if count firms with [ HEI ] < nHEIs [ make-HEI ] ; count the HEIs and create one if there are 
                                               ;     too less
    ask firms [ set partners [] ]
    if plotting [ layout-firms ]
    
    
    if plotting [ set activity "Doing research" ]
    ;set selling false
    ask firms [         
        without-interruption [
             if collaboration [ collaborate ]
             do-research 
             manufacture
             ;if not HEI [ pay-taxes ]
             pay-taxes
             if HEI
               [
               repeat 1 [ pay-taxes ]
               ]
             set sales 0
             set customers []
             ]
        ]

    if plotting [ set activity "Selling to end users" ]
    ask firms with [ not HEI ] [ sell-to-end-users ]
    if plotting [ set activity "Finding suppliers" ]
    ask firms with [ not HEI ] [ find-suppliers ]
    if plotting [ set activity "Buying components" ]
    ask firms with [ not HEI ] [ buy ]
    if plotting [ set activity "Adjusting prices" ]
    ask firms with [ not HEI ] [ 
        take-profit
        adjust-price
        ]
    if plotting [ set activity "Creating networks and start-ups" ]

    create-networkagents
      
    create-start-ups
    create-spin-offs

   
    if plotting [ set activity "Updating plots" ]
    if plotting [ show-plots ]
    ask networks [ distribute-network-profits ]
    ask firms with [ not HEI ] [ set-counters ]
    ask firms [ do-admin ]
  
    let nHEIcoops 0
    ask firms with [ HEI ] [ set nHEIcoops nHEIcoops + length partners ]       
    let firm-HEI-coops 0
    let firm-count 0
    ask firms with [ not HEI ] [ 
       let HEI-partner false
       set partners remove nobody partners 
       foreach partners [ if [ HEI ] of ? [ set HEI-partner true ] ]
       ifelse HEI-partner
         [ set firm-HEI-coops firm-HEI-coops + length partners
           set firm-count firm-count + 1
         ]
         [ set firm-HEI-coops firm-HEI-coops ]
       ]   
    data-write 
    set step step + 1
end    

; a kene is a set of triples of capability, ability and expertise.  These are held as 
; three arrays (vectors).  The length of the kene is proportional
; to the capital of the firm.

to make-kene
    let nKenes ( initial-capital / capital-knowledge-ratio ) + 1 
    
    if HEI [ set nKenes nKenes + 5 ]   
    
    repeat nKenes [ set capabilities fput (random nCapabilities) capabilities ]
    
    set capabilities sort remove-duplicates capabilities
    
    while [ length abilities < length capabilities ] [
        set abilities fput (random 10) abilities
        set expertises fput ((random 10) + 1) expertises
        ]
end         
 
; an innovation hypothesis is a vector of locations in the kene.  So, for example, an IH
; might be [1 3 4 7], meaning the second (counting from 0 as the first), fourth, fifth 
; and eighth triple in the kene.  The IH cannot be longer than the length of the kene, 
; but is of random length.  

to make-innovation-hypothesis
    let kene-length 0
  let ih-length 0
  let location 0
   
    set ih []
    set kene-length length capabilities
    set ih-length min (list ((random max-ih-length) + 2) kene-length)
    while [ ih-length > 0 ] [
        set location random kene-length
        if not member? location ih [
            set ih fput location ih
            set ih-length ih-length - 1
            ]
        ]
    set ih sort ih 
    set research-direction "random"   
end

; set up the firm's advert, which is the list of capabilities in its innovation
; hypothesis

to make-advert
    set advert map [ item ? capabilities ] ih
end
 
; a firm's product is computed from its innovation hypothesis, using the capabilities and 
;    abilities       
to make-product
    set product map-artefact ih
end 

; the quality of a firm's product is computed from the abilities and expertise in its 
; innovation hypothesis: it is the sum (modulo 10) of the product of the abilities and 
; (1 - e to the power of the corresponding expertise level)

to make-quality
    set quality 
        (reduce [?1 + ?2]   
            (map [ (item ? abilities) * (1 - exp (0 - (item ? expertises))) ] 
                    ih))
                mod 10
end

; the mapping from innovation hypothesis to product number
; the product is the sum (modulo the total number of products) of the product of the 
; capabilities and the abilities in the innovation hypothesis
to-report map-artefact [ locations ]
    report  
        (reduce [?1 + ?2]   
            (map [ (item ? capabilities) * (item ? abilities) ] locations))
                mod nProducts
end 
    
; a firm has a random number of inputs between 3 and nInputs, but not more than the
; length of its innovation hypothesis
; This procedure calculates what products a firm needs for its inputs.
; This is done by chopping the innovation hypothesis into sections, one for each input, and
; then mapping the section into a product using the standard mapping function

to make-inputs
    let width 0
  let nIn 0
  let i 0
  let finish 0
  let inputLocations 0
  
    set inputs []
    set nIn min list length ih max list 3 (random nInputs + 1)    ; the number of inputs
    set width int (length ih / nIn)        ; the width of each section of the IH
        set i 0
        repeat nIn [                           ; for each input, copy the section of the IH 
                                               ;   into inputLocations
           set finish i + width
           set inputLocations []
           while [i < finish] [
              set inputLocations fput item i ih inputLocations
              set i i + 1
              ]
           ; and map from section to product number
           set inputs fput map-artefact inputLocations inputs 
        ]
;        show "Inputs " + inputs + "Product " + product
        set inputs remove-duplicates inputs    ; all inputs must be of different products
        set inputs remove product inputs       ; and must differ from this firm's product
        if length inputs = 0 [ exit ]          ; if this is not an effective producer, give up.
end

; find some partners to form partnerships with

to collaborate
    find-partners
    set partners remove nobody partners
    learn-from-partners
    pay-tax collaboration-tax * length partners
end

; try up to 5 times to find partners to collaborate with, looking first at previous 
; partners and then at suppliers, customers and finally others.  For each partner
; found, tell the partner that I am now a partner and draw an edge on the network
; diagram

to find-partners
    let candidates 0
    if HEI [ stop ]                ; disable HEI-HEI-partnerships
    if plotting [ disconnect-edges "partner" ]
    set candidates (sentence previous-partners suppliers customers)
    let HEI-candidates list one-of firms with [ HEI ] one-of firms with [ HEI ]
    while [ length candidates < 5 ]
        [ set candidates lput one-of firms candidates ]
    set candidates remove-duplicates candidates
    while [ length candidates > 5 ]
        [ set candidates but-last candidates ]
    set partners filter [ compatible? ? ] remove nobody sentence candidates HEI-candidates
    foreach partners [ 
      set [partners] of ? remove-duplicates fput self [partners] of ?
      if plotting [ connect-edge ? "partner" ] ]
end 

; reports true or false according to whether the possible partner is sufficiently attractive, 
; according to the current partnership strategy

to-report compatible? [ possible-partner ]
    let attractiveness 0
                  
    ; reject impossible potential partners
    if possible-partner = self or possible-partner = nobody or not is-agent? possible-partner [ report false ]
    ; a partner cannot already be a member of my network
    if is-agent? network and network != nobody and member? possible-partner [members] of network [ report false ]
    ifelse partnership-strategy = "conservative"
        [ set attractiveness (length intersection advert [advert] of possible-partner) /
                              (min list length advert length [advert] of possible-partner) ]
        [ ifelse (length intersection advert [advert] of possible-partner) >= 1 
          [ set attractiveness (length set-difference   advert [advert] of possible-partner) /
                                (length advert + length [advert] of possible-partner) ]
          [ set attractiveness 0 ]
         ]
    report attractiveness > attractiveness-threshold 
end

; reports the set intersection of the lists a and b, treated as sets

to-report intersection [ set-a set-b ]
    let set-c 0
  
    set set-c []
    foreach set-a [ if member? ? set-b [ set set-c fput ? set-c ] ]
    report set-c
end

; reports the set difference of lists a and b, treated as sets
to-report set-difference [ set-a set-b ]
    let set-c 0
  
    set set-c intersection set-a set-b
    set set-b remove-duplicates sentence set-a set-b
    foreach set-c [ if member? ? set-b [ set set-b remove ? set-b ] ] 
    report set-b
end

; obtain capabilities from partners.  The capabilities that are learned are those 
; from the partners' innovation hypothesis.

to learn-from-partners
    foreach partners [ merge-capabilities ? ]
end

; for each capability in the other's innovation hypothesis, if it is new to me, 
; add it (and its ability) to my kene (if I have sufficient capital), and make   
; the expertise level 1. For each capability that is not new, if the other's  
; expertise level is greater than mine, adopt his ability and expertise level, 
; otherwise do nothing.

to merge-capabilities [ other_agent ]
   let capability 0
   let my-position 0
   let me-old 0
   let me-new 0
   
   set me-old sqrt ( sum ( map [ ?1 * ?2 ] expertises abilities ) )
   foreach [ih] of other_agent [
        set capability item ? [capabilities] of other_agent
        ifelse member? capability capabilities
            ;  capability already known to me
            [ set my-position position capability capabilities
              if item my-position expertises < item ? [expertises] of other_agent
                  [ set expertises replace-item my-position expertises 
                                                  item ? [expertises] of other_agent 
                    set abilities replace-item my-position abilities 
                                                  item ? [abilities] of other_agent] ]
            ; capability is new to me; adopt if I have 'room'
            [ if (length capabilities) < ((capital / capital-knowledge-ratio) + 1) [
                  set capabilities sentence capabilities capability
                  set abilities sentence abilities item ? [abilities] of other_agent
                  set expertises sentence expertises 1 ] ]
        ]
   ;show ( word ( map [ ? * ? ] expertises abilities ) expertises abilities ) 
   set me-new sqrt ( sum (map [ ?1 * ?2 ] expertises abilities) )
   if knowledge-flow-list = 0 [ set knowledge-flow-list [] ]
   set knowledge-flow-list fput (word "," who "," [who] of other_agent "," floor ( me-new - me-old ) )  knowledge-flow-list
   set knowledge-flow knowledge-flow + me-new - me-old
   ;show (word "from" [who] of other_agent " " me-new " " me-old " " floor ( sqrt ( me-new - me-old ) ) )
end
 
; firms normally do incremental research, if their last innovation has been 
; unsuccessful.  However, if their capital has almost all gone, they
; do radical research.

to do-research
    if last-reward < success-threshold [
        ifelse capital <= low-capital-threshold
            [ do-radical-research ]
            [ do-incremental-research ]
        ]
end
    
; adjust an ability up or down.  If this is the second or subsequent time of doing incremental
; research, and the last reward was positive (although less than success-threshold, since if it
; were above that, the firm would continue to produce the last innovation, rather than do 
; research), make a change to the same ability in the same direction.  
; If the first time or the research proved unsuccessful (the firm made a loss and last-reward  
; is 0), choose a capability from the innovation hypothesis at random and alter the ability 
; in a random direction by one unit. 

; research-direction is initially "random".  It is set here to "down" or "up" to show  
; direction of research.  If an ability has been explored to the limit (reached 0 or 10),
; the direction is reset to random.

to do-incremental-research
    let new-ability 0
  
    if last-reward <= 0 [ set research-direction "random" ]
    if research-direction = "random"  [
        set ability-to-research random length ih
        ifelse (random 2) = 1
            [ set research-direction "up" ]
            [ set research-direction "down" ]
        ]
    set new-ability item ability-to-research abilities
    ifelse research-direction = "up" 
        [ set new-ability new-ability + 1 ]
        [ set new-ability new-ability - 1 ]
    if new-ability < 0  [ set new-ability 0   set research-direction "random"]
    if new-ability > 10 [ set new-ability 10  set research-direction "random"]
    set abilities replace-item  ability-to-research abilities new-ability
    pay-tax incr-research-tax
end        

; radical research just means 
; a. randomly changing one capability for a new value and then
; b. constructing a new innovation hypothesis from the kene

to do-radical-research
    let capability-to-mutate 0
  let new-capability 0
  
    set capability-to-mutate (random length capabilities)
    set new-capability (random nCapabilities) ; find a capability that is new to this firm
    while [ member? new-capability capabilities ] 
        [set new-capability (random nCapabilities) ]
    set capabilities 
        replace-item capability-to-mutate capabilities new-capability
    make-innovation-hypothesis
    make-advert
    pay-tax radical-research-tax
end

; calculate the product that will be made, the inputs that will be required and 
; the quality of the product (manufacture may the wrong word, it's more like product
; development).  Adjust my expertise.

to manufacture
  if not HEI [
    make-product
    make-inputs
    make-quality
    ]
    adjust-expertise
    
end

; raise the expertise level by one (up to a maximum of 10) for capabilities that 
; are used in the innovation, and decrease by one for capabilities that are not.  
; If an expertise level has dropped to zero, the capability is forgotten.

to adjust-expertise
    let location 0
  let expertise 0
  
    set location 0
    while [ location < length capabilities ] [
        set expertise item location expertises
        ifelse member? location ih 
             [ if expertise < 10 
                 [ set expertises replace-item location expertises (expertise + 1) ] ]
             [ ifelse expertise > 0  
                 [ set expertises replace-item location expertises (expertise - 1) ]
                 [ forget-capability location ] ]
        set location location + 1
    ]       
end

; remove the capability, ability, expertise at the given location of the kene.  
; Warning; all hell will break loose if the capability that is being forgotten is included 
; in the innovation hypothesis (but no test is done to check this)

to forget-capability [location ]
    set capabilities remove-item location capabilities
    set abilities remove-item location abilities
    set expertises remove-item location expertises
    adjust-ih location
end

; reduce the values (which are indices into the kene) in the innovation hypothesis to 
; account for the removal of a capability.  Reduce all indices above 'location' by one

to adjust-ih [ location ]
    let i 0
  let elem 0
  
    set i 0
    while [ i < length ih ] [
        set elem item i ih
        if elem > location [ set ih replace-item i ih (elem - 1 ) ] 
        set i i + 1
        ]
end

; If the firm is producing a consumer product, sell it to the end user

to sell-to-end-users
    if selling? and product > nProducts - end-products [
        set capital capital + final-price
        set sales sales + final-price
        set nCustomers nCustomers + 1
;        show "Selling to end-user " + product
        ]
end

; Find suppliers able to provide all my inputs.
; If the input is a raw material, suppied from outside the system, note that, else
; each supplier is chosen using the criteria: 
;    (a) it provides the right product and has it available to sell, 
;         and if there are more than one
;    (b) at the cheapest price, and if there are more than one,
;    (c) at the highest quality, and if there are more than one
;    (d) chosen at random
; The suppliers are put into 'suppliers', a list of firms.  If there is no supplier 
; that satisfies these criteria for one or more inputs, then the supplier for that 
; input is recorded as 'nobody'

to find-suppliers
    let possible-suppliers 0
    let a-cheapest-supplier 0
    let cheapest-suppliers 0
    let supplier 0
  
    set suppliers []
    set total-cost 0
    (foreach inputs [
        ifelse ? < raw-materials [
            set supplier "raw-material"
            set total-cost total-cost + raw-cost
            ]
            [
            set supplier nobody
            set possible-suppliers firms with [product = ? and selling? ]
            set a-cheapest-supplier min-one-of possible-suppliers [ price ]
            if a-cheapest-supplier != nobody [
               set cheapest-suppliers possible-suppliers with [price = [price] of a-cheapest-supplier]
               set supplier max-one-of cheapest-suppliers [quality]
               set total-cost total-cost + [price] of supplier
               ]
            ]
        set suppliers lput supplier suppliers
        ]  
    ) 
    ; no customers yet
    set nCustomers 0           
end

; Buy the required inputs from suppliers.
; If there is no supplier available for one or more inputs, do nothing.
; If the total cost of the inputs is greater than or equal to the price I have set for 
;    my product do nothing.
; For each input, deduct price from my capital and pay the supplier
; If all inputs have been obtained, note that I have been able to produce 
;    (the product will go on sale in the next round) 
; If the input is a raw-material, do much the same

to buy
    if plotting [ disconnect-edges "supplier" ]
    set selling? false
    if not member? nobody suppliers and total-cost < price [
        foreach suppliers [
             ifelse ? = "raw-material" 
                 [ set capital capital - raw-cost ]
                 [ 
                   set capital capital - [price] of ?
                   set [capital] of ? [capital] of ? + [price] of ?
                   set [nCustomers] of ? [nCustomers] of ? + 1
                   set [customers] of ? fput self [customers] of ?
                   set [sales] of ? [sales] of ? + [price] of ?
;                  show word "Buying " [ product ] of ? " from " ? " at price of " [ price ] of ? 
                   if plotting [ connect-edge ? "supplier" ]
                 ]
        ]
        set selling? true
    ]
end

; calculate the profit I have made from selling my product

to take-profit
    set last-reward 0
    if nCustomers > 0 and price > total-cost [
        set last-reward nCustomers * (price - total-cost)
        ]
end
   
; Adjust the price of my product to respond to market conditions: 
; If I made a product (I have suppliers for all inputs) then ensure that my selling 
;   price will at least equal the total cost of production
; If in the last round I sold a product, increase the price by 10% for every customer
; If there were no buyers, reduce the price by 10%
; If I did not manage to make a product, because one or more inputs were missing, do nothing

to adjust-price
    if not member? nobody suppliers [
        if price < total-cost [ set price total-cost ]
        ifelse nCustomers > 0 
            [ set price price * (1 + (0.1 * nCustomers))]
            [ set price price * 0.9 ]
    ]
end            

; if the firm's last innovation was successful and they are not already a member of 
; a network and they have some partners and they have enough capital, initiate the 
; formation of a network and add myself and my partners as the initial members

to create-networkagents
    ask firms [
        without-interruption [
             set partners remove nobody partners
             if not is-agent? network and length partners > 0 and 
                 last-reward > success-threshold and capital > initial-capital and
                 length filter [ not is-agent? [network] of ? ] partners > 0 
                     [ create-network ]
        ]
    ]
end

; create a network agent founded on myself as the first member and add my partners as members.
; a network consist of a network agent with an associated network firm.  The latter does all
; the work of innovation and production for the network.

to create-network
    let the-network 0
  
    set the-network make-network
    add-myself-and-partners the-network
;    show the-network + "firm= " + [firm] of the-network + " Members=" + [members] the-network
    ask [firm] of the-network [
        make-innovation-hypothesis
        make-advert
        manufacture
        ]
end

; make an 'empty' network (a new network object and a new firm object)

to-report make-network
    let founder 0
  let net 0
  
    set founder self
    hatch 1 [    ; create a network agent
        set breed networks
        hide-turtle
        set members []
        hatch 1 [    ; create a firm for the network
            set breed firms
            set [firm] of myself self    ; tell network about its firm
            initialise-firm
            set network-firm true
            set network myself         ; tell network firm about its network
            copy-ih-capabilities founder    ; the network's firm starts with a 
                                            ;  copy of the capabilities from the 
                                            ;  founder's kene
            show-turtle
            ]
        set net self
        ]
    report net
end

; copy the kene triples corresponding to the innovation hypothesis
;   from the source to this firm

to copy-ih-capabilities [ source ]
    let pos 0
  
    foreach [ih] of source [
        set capabilities fput item ? [capabilities] of source capabilities 
        set abilities fput item ? [abilities] of source abilities 
        set expertises fput item ? [expertises] of source expertises
        ]
end 

; if I have enough capital and am not already a member, add myself to the network, and 
; then add all my partners to the network (and so on recursively)

to add-myself-and-partners [ the-network ]
    if not is-agent? network and not member? self [members] of the-network 
            and capital > initial-capital / (1 + length [members] of the-network) [
;        foreach [ members ] of the-network [ if plotting [ connect-edge ? "network" ] ]
        if plotting [ connect-edge [firm] of the-network "network" ]
        set [members] of the-network fput self [members] of the-network
        set [network] of self the-network
        merge-capabilities [firm] of the-network 
        foreach partners [
            if ? != nobody [ ask ? [add-myself-and-partners the-network ] ]
            ]
        ]
end

; no comment needed!

to pay-taxes
    pay-tax depreciation        ; annual depreciation
end

; the money paid just disappears.  Later it may be transferred to end-users, to keep it 
;   in circulation

to pay-tax [ amount ]
    set capital capital - amount
end

; If the network has been successful, distribute all the profits of the network 
; above the initial capital equally to all the network members
; Alternatively if the network has gone bankrupt, dissolve the network

to distribute-network-profits
     if [capital] of firm > initial-capital [
         foreach members [ 
             set [capital] of ? [capital] of ? + 
                         (([capital] of firm - initial-capital) / length members )
             ]
         ]
     if [capital] of firm < 0 [ dissolve-network ]
end

; unhook each member from the network and kill both the network agent and the associated
;   firm agent

to dissolve-network
     foreach members [
         set [network] of ? []
         if plotting [ ask ? [ disconnect-edges "network" ] ]
         ]
     ask firm [ die ]    
     die
end

; some final tidying up at the end of every round

to set-counters
    let n 0        
    let HEI-coop false
    
    if network != nobody and network != [] and is-agent? network[
      foreach [members] of network [if [HEI] of ? [set HEI-coop true]]
    ]
    set partners remove nobody partners
      
    foreach partners [
      if [HEI] of ? [ set n n + 1 ]
      ]
       
    if n > 0 [ set HEI-coop true ]
      
    ifelse HEI-coop [
      set time-in-HEI-coop time-in-HEI-coop + 1
      set time-since-HEI-coop 0
      ]  
      [
      set time-in-HEI-coop 0
      set time-since-HEI-coop time-since-HEI-coop + 1
      ]
end

to do-admin            
    ; dissolve this round's partnerships, but remember the partners for the future
    set previous-partners remove-duplicates sentence previous-partners partners
    set age age + 1
    
    ; die if bankrupt
    if capital < 0 [ exit ]
end

; remove myself from the simulation, making sure that I am no longer a member of any network etc.

to exit
    ; remove the firm from its network (if any).  If the network has zero or one 
    ;  members, also kill the network
    if is-agent? network and network != nobody [
        set [members] of network remove self [members] of network
        if length [members] of network <= 1 or network-firm [
            ask network [ dissolve-network ]
            ]
        ] 
    set age-at-death lput age age-at-death 
    while [ length age-at-death > 100 ] [
      set age-at-death but-first age-at-death
      ]   
    die 
end

; start-ups are clones of the most successful firm, modified in two ways:
; they get only the standard starting capital
; they get their kene trimmed to the length appropriate to their capital


to create-start-ups
  
    ifelse count firms > 0 and [last-reward] of max-one-of firms [ last-reward ] > reward-to-trigger-start-up
      [ set nStart-ups  max (list 1 ( log ([last-reward] of max-one-of firms [ last-reward ] - reward-to-trigger-start-up) 10 / 2 ) ) ]
      [ set nStart-ups 0 ]
    repeat nStart-ups [ make-start-up ]
    ;if count firms > 0 and
    ;    [last-reward] of max-one-of firms [ last-reward ] > reward-to-trigger-start-up
    ;        [ 
    ;          repeat nStart-ups [ make-start-up ]
    ;        ]
end

to create-spin-offs
 if count firms > 0 [
   let HEI_list []
   ask firms with [HEI = true] [
     let HEI_succ  0.0
     foreach partners [if [last-reward] of ? > success-threshold [set HEI_succ HEI_succ + 1.0]]
     ;show (word "succ" HEI_succ );"partners" partners "advert" advert)
     if (length partners) != 0 [set HEI_succ HEI_succ / (length partners)]
     if HEI_succ >= HEI-success-threshold / 100. [
       set HEI_list fput self HEI_list
     ]
   ]
   foreach HEI_list [
     make-spin-off ?
     set [ capital ] of ? [ capital ] of ? + spin-off-Bonus
   ]
   set nSpin-offs length HEI_list
 ]
end

; create one HEI, called by the go-procedure to garantuee a constant #HEI

to make-HEI 
    create-firms 1 [
        ;show ( word "creating trutle" who " as HEI ")
        set HEI true
        initialise-firm
        set HEI true
        set shape "arrow"
        set color 125
        set heading 90
        make-kene
        make-innovation-hypothesis
        make-advert
        set product -1
        manufacture
        
    ]
 end

; clone the firm with the largest reward

to make-start-up
    create-firms 1 [
        initialise-firm
        clone-kene max-one-of firms with [ self != myself ] [ last-reward ]
        make-innovation-hypothesis
        make-advert
        manufacture
    ]
end

to make-spin-off [other-agent]
  ;show "creating spin-off"
  
  create-firms 1 [
    initialise-firm
    clone-kene other-agent
    set previous-partners []
    let kene-pos 0
    repeat (length [expertises] of other-agent) [
      if (item kene-pos ([expertises] of other-agent)) != 0[
        set ([expertises] of other-agent) replace-item kene-pos ([expertises] of other-agent) 
                                          ((item kene-pos ([expertises] of other-agent)) - 1)                                          
      ]                                    
      set kene-pos kene-pos + 1
    ]
    set previous-partners other-agent
    let k_replace (random length [expertises] of other-agent)
    let new-capability (random nCapabilities)
    while [ member? new-capability [capabilities]  of other-agent] 
        [set new-capability (random nCapabilities) ]
    set ([capabilities] of other-agent) replace-item k_replace ([capabilities] of other-agent) new-capability 
    set ([abilities] of other-agent) replace-item k_replace ([abilities] of other-agent) (random 10)
    set ([expertises] of other-agent) replace-item k_replace ([expertises] of other-agent) ((random 10) + 1)
    
    make-innovation-hypothesis
    make-advert
    manufacture
  ]
end

; cloning a kene involves copying the triples in its innovation hypothesis, 
; but chopping it to a length suitable for the capital of the firm receiving it

to clone-kene [ firm-to-clone ]
    let ih-pos 0
  let triple-pos 0
  
    set ih-pos 0
    repeat min list (length [ih] of firm-to-clone) ((capital / capital-knowledge-ratio) + 1) [
        set triple-pos item ih-pos [ih] of firm-to-clone
        set capabilities fput (item triple-pos [capabilities] of firm-to-clone) capabilities
        set abilities fput (item triple-pos [abilities] of firm-to-clone) abilities
        set expertises fput (item triple-pos [expertises] of firm-to-clone) expertises
        set ih-pos ih-pos + 1
        ]
end         
  
; display some plots  

to show-plots
  let sum-capital 0
  let turnovers 0
  let avg 0

  if plotting [ 
    
    layout-firms 
    
    set-current-plot "Concentration"
    
    set-current-plot-pen "Herfindahl"
    set sum-capital sum [ capital ] of firms
    plot 100 * sum [ (capital / sum-capital ) ^ 2 ] of firms
;    set-current-plot-pen "Avg Turnover"
;    set turnovers values-from firms with [ sales > 0 ] [ sales ]
;    if sum turnovers > 0 
;        [ plotxy step 100 * mean turnovers / max turnovers ]
        
    set-current-plot "Capital"
    if count firms with [ capital > 0  and not HEI ] > 1 [
      set-current-plot-pen "Average"
      set avg mean [ log capital 10 ] of firms with [ capital > 0 and not HEI ] 
      plot avg
      set-current-plot-pen "+1 St Dev"
      plot avg + standard-deviation [ log capital 10 ] of firms with [ capital > 0 and not HEI ] 
     
      set-current-plot-pen "-1 St Dev"
      plot avg - standard-deviation [ log capital 10 ] of firms with [ capital > 0 and not HEI ] 
      ]
    
    
    set-current-plot "Collaboration"
   
    set-current-plot-pen "In partnership"
    if count firms with [ not HEI ] > 0 [
      plot 100 * sum [ in-partnership ] of firms with [ not HEI ]  / count firms with [ not HEI ]
      set-current-plot-pen "In network"
      plot 100 * count firms with [ is-agent? network ] / count firms with [ not HEI ]
      set-current-plot-pen "Connectedness"
      plot 100 * ((sum [ length partners ] of firms / 2) + 
                  (perm sum [ length members ] of networks ) ) /
                  ( perm count firms )
      ]
    
    if any? networks [ plot-degree-distribution ]        

    set-current-plot "Population"
    set-current-plot-pen "Firms"
    plot count firms with [ not HEI ]
    set-current-plot-pen "Networks"
    plot count networks 
    
    set-current-plot "Dynamics"
    set-current-plot-pen "Successes"
    plot count firms with [ last-reward > success-threshold ]
    set-current-plot-pen "Start-ups"
    plot count firms with [ age = 0 and step != 0 and not network-firm and not HEI ]
    
    set-current-plot "Transactions"
    set-current-plot-pen "Firms selling"
    if count firms with [ capital > 0 and not HEI ] > 0 [
      plot 100 * count firms with [ selling? and not HEI ] / count firms with [ not HEI ]
      set-current-plot-pen "Sales"
      plot 100 * sum [ nCustomers ] of firms  / sum [ length inputs ] of firms with [ not HEI ]
    ]
    
    set-current-plot "Networks"
    if count networks > 0 [
      set-plot-x-range 1  max list 20 1 + max  [length members] of networks
      ;histogram-from networks [ length members ]
      histogram [ length members ] of networks
      ]
    set-current-plot "Partners"
    ;histogram-from firms [length partners] 
    histogram [ length partners ] of firms with [ not HEI ]
    
    set-current-plot "knowledge-flow-plot"
    set-current-plot-pen "knowledge-flow"
    plot knowledge-flow
    
    set-current-plot "knowledge-plot"
    set-current-plot-pen "knowledge"
    let knowledge 0
    ask firms with [ capital > 0 and not HEI ] [ set knowledge knowledge + floor ( sqrt( sum ( map [ ?1 * ?2 ] expertises abilities ) ) ) ]
    plot knowledge
    set-current-plot-pen "1st Dev knowledge"
    plot standard-deviation [ sqrt( sum ( map [ ? * ? ] abilities expertises ) ) ] of firms with [ capital > 0 and not HEI ] * 100
    
  ];closing for 'if plotting'
end

;Add-on for Export of all data 30.08.2008 by Cedrick Ansorge


to header-write
    file-print (word "#HEADER_LINES 50" )
    file-print (word "#This File was generated by the SKIN-Model extenden by Hannes Brauckmann and Cedrick Ansorge with NetLogo")
    file-print ("#")
    file-print (word "#The file contains four lines for each time step These lines are" )
    file-print (word "#   1. VARI_LINE" )
    file-print (word "#      It contains as a list of floating/integer numbers (without separators)" ) 
    file-print (word "#      Step Firms-selling Sales Start-ups Successes Firms Networks In-Partnership In-Network Connectedness Average-capital Average-capital+1stDev Average-capital-1stDev Herfindahl-Index knowledge-flow knowledge 1stDev-knowledge avg-length-partners-coop-with-HEI avg-length-partners-not-coop-with-HEI mean-average-at-death nSpin-offs" )
    file-print (word "#      In generals -1 here indicates a missing or unsensible value" )
    file-print (word "#   2. EDGE_LINE" )
    file-print (word "#      This Line contains as alist of comma-seperated integers the knowledge-flows." )
    file-print (word "#      The reading convention is, that each threee values represent firm1,firm2,flow" )
    file-print (word "#      Whreas firm1, firm2 are firm-IDs and flow is the knowledge-increase of firm1 due to cooperation with firm to" )
    file-print (word "#   3. SICO_LINE" )
    file-print (word "#      This line contains the average last reward of firms that DID NOT cooperate with a HEI since n Steps whereas" )
    file-print (word "#      n is the occurence in this line(counted from zero)." )
    file-print (word "#      -1 indicates that there where no firms with that value" )
    file-print (word "#   4. INCO_LINE" )
    file-print (word "#      This line contains the average last reward of firms that DID cooperate with a HEI since n Steps whereas" )
    file-print (word "#      n is the point of occurence in this line (counted from zero)." )
    ; olist2 contains the average last-rewards of firms for each value of time-since-HEI-coop < 100
  
    ; olist3 contains the average last-rewards of firms for each value of time-in-HEI-coop < 100
    file-print (word "#EXPERIMENT_SETUP           "date-and-time)
    file-print (word "#PLOTTING                   "plotting )
    file-print (word "#INITIAL_CAPITAL            "initial-capital )
    file-print (word "#NPRODUCTS                  "nProducts )
    file-print (word "#RAW_MATERIALS              "raw-materials )
    file-print (word "#END_PRODUCTS               "end-products )
    file-print (word "#NFIRMS                     "nFirms )
    file-print (word "#N_BIG_FIRMS                 "n-big-firms )
    file-print (word "#NINPUTS                    "nInputs )
    file-print (word "#NHEIS                      "nHEIS )
    file-print (word "#MAXPRICE                   "maxPrice )
    file-print (word "#REWARD_TO_TRIGGER_START_UP "reward-to-trigger-start-up )
    file-print (word "#MAX_IH_LENGTH              "max-ih-length )
    file-print (word "#NCAPABILITIES              "nCapabilities )
    file-print (word "#ATTRACTIVENESS_THRESHOLD   "attractiveness-threshold )
    file-print (word "#SUCCESS_THRESHOLD          "success-threshold )
    file-print (word "#HEI_SUCCESS_THRESHOLD      "HEI-success-threshold )
    file-print (word "#LOW_CAPITAL_THRESHOLD      "low-capital-threshold )
    file-print (word "#CAPITAL_KNOWLEDGE_RATIO    "capital-knowledge-ratio )
    file-print (word "#INCR_RESEARCH_TAX          "incr-research-tax )
    file-print (word "#COLLABORATION_TAX          "collaboration-tax )
    file-print (word "#RAW_COST                   "raw-cost )
    file-print (word "#FINAL_PRICE                "final-price )
    file-print (word "#COLLABORATION              "collaboration )
    file-print (word "#PARTNERSHIP_STRATEGY       "partnership-strategy )
    file-print (word "#OPEN_SYSTEM                "open-system )
end

to data-write
    let olist []
   
    ; olist contains 
    ; Step Firms-selling Sales Start-ups Successes Firms Networks In-Partnership In-Network Connectedness 
    ;     Average-capital Average-capital+1stDev Average-capital-1stDev Herfindahl-Index 
    ;     knowledge-flow knowledge 1stDev-knowledge 
    ;     avg-length-partners-coop-with-HEI avg-length-partners-not-coop-with-HEI
    ;     mean-average-at-death
    
    ; olist2 contains the average last-rewards of firms for each value of time-since-HEI-coop < 100
  
    ; olist3 contains the average last-rewards of firms for each value of time-in-HEI-coop < 100
   
    set olist lput step olist
    if count firms with [ capital > 0 and not HEI ] > 0 [
      ;Firms selling
      set olist lput ( 100 * count firms with [ selling? and not HEI ] / count firms with [ not HEI ]) olist
    
      ;Sales
      set olist lput ( 100 * sum [ nCustomers ] of firms / sum [ length inputs ] of firms with [ not HEI ]) olist
    
      ;Start-ups
      set olist lput nStart-ups olist 
      ;show ( word nStart-ups "Start-ups" )
      ]
      
    ;Successes
    let successes count firms with [ last-reward > success-threshold ]
    set olist lput successes olist
    
    ;Firms
    set olist lput ( count firms with [ not HEI ] ) olist
    
    ;Networks
    set olist lput ( count networks ) olist
    
    ;Collaboration
    if count firms with [ not HEI ] > 0 [
      ;In partnership
      set olist lput ( 100 * sum [ in-partnership ] of firms / count firms with [ not HEI ]  ) olist
      ;In network
      set olist lput ( 100 * count firms with [ is-agent? network ] / count firms with [ not HEI ] ) olist 
      ;Connectedness
      set olist lput ( 100 * ((sum [ length partners ] of firms / 2) + 
                   (perm sum [ length members ] of networks ) ) /
                     ( perm count firms) ) olist
    
    ;Average Capital
      if count firms with [ not HEI and capital > 0 ] > 0 [ 
        let avg mean [ capital ] of firms with [ capital > 0 and not HEI ] 
        set olist lput avg olist
      
        if count firms with [ capital > 0 and not HEI ] > 1 [ 
          ;+1st Dev
          set olist lput ( avg + standard-deviation [ capital ] of firms with [ capital > 0 and not HEI ] ) olist
          ;-1st Dev
          set olist lput ( avg - standard-deviation [ capital ] of firms with [ capital > 0 and not HEI ] ) olist
          ]
        ]
      ]
    ;Herfindahl
    if count firms with [ capital > 0 ] > 0 [
      let sum-capital sum [ capital ] of firms with [ not HEI ]
      let herfindahl sum [ (capital / sum-capital ) ^ 2 ] of firms with [ not HEI ]
      set olist lput ( 100 * herfindahl ) olist
    ;knowledge-flow
      set olist lput knowledge-flow olist
    ;knowledge
      let knowledge 0
      ask firms with [ capital > 0 and not HEI ] [ set knowledge knowledge + floor ( sqrt ( sum ( map [ ? * ? ] capabilities abilities ) ) ) ]
      set olist lput (knowledge / count firms with [ capital > 0 and not HEI ]) olist
      ;1st Dev knowledge
      set olist ( lput ( ( standard-deviation [ floor ( sqrt ( sum ( map [ ? * ? ] abilities expertises ) ) ) ] 
      of firms with [ capital > 0 and not HEI ] ) / count firms with [ capital > 0 and not HEI ] ) olist )
    ]
     
    let lengths-partners-with-HEI []
    let lengths-partners-without-HEI []
    
    ask firms with [ not HEI ] [
       let HEI-partner false
       set partners remove nobody partners
       foreach partners [
          if [HEI] of ? [ set HEI-partner true ]
          ]
       ifelse HEI-partner 
          [ set lengths-partners-with-HEI lput length partners lengths-partners-with-HEI ]
          [ set lengths-partners-without-HEI lput length partners lengths-partners-without-HEI ]
       ]
     ifelse length lengths-partners-with-HEI > 0 
       [ set olist lput mean lengths-partners-with-HEI olist  ]
       [ set olist lput -1 olist ]
     ifelse length lengths-partners-without-HEI > 0 
       [ set olist lput mean lengths-partners-without-HEI olist  ]
       [ set olist lput -1 olist ]     
     ;show ( word "with " m " without" o )
     
    ifelse length age-at-death > 0 
      [ set olist lput ( mean age-at-death ) olist ]
      [ set olist lput -1 olist ]
    
    set olist lput nSpin-offs olist
    ;show ( word nSpin-offs "Spin-offs" )


    ;
    let nBins 100
    let n 0
    let olist2 []
    let value 0
    repeat nBins [ 
      ifelse count firms with [ time-since-HEI-coop = n ] > 0 
        [ set olist2 lput mean [ last-reward ] of  firms with [ time-since-HEI-coop = n ]  olist2 ]
        [ set olist2 lput -1 olist2 ]
      set n n + 1
      ]
    ;show ( word "olist2:" olist2 )

    let olist3 []
    
    set n 0
    repeat nBins [
      ifelse count firms with [ time-in-HEI-coop = n ] > 0
        ;[ set olist3 lput mean [last-reward] of firms with [ time-in-HEI-coop = n ] olist3 ]
        [ set olist3 lput count firms with [ time-in-HEI-coop = n and last-reward > success-threshold ] olist3]
        [ set olist3 lput 0 olist3 ]
      set n n + 1
      ]
    ;show ( word "olist3 n:" n )
    ;show ( word "olist3:" olist3  )
    file-print ( word "VARI_LIST" olist )
    file-print ( word "NODE_LIST" knowledge-flow-list )
    file-print ( word "SICO_LIST" olist2 )
    file-print ( word "INCO_LIST" olist3 )
    
    ;file-print ( word olist )
    ;file-print ( word knowledge-flow-list )
    ;file-print ( word olist2 )
    ;file-print ( word olist3 )
    
end


; report whether or not I am in a partnership

to-report in-partnership
    ifelse length partners > 0 [ report 1 ] [ report 0 ]
end

to-report perm [ num ]
  report num * (num - 1 ) / 2
end

;
; plot log(number of members) by log(frequency of networks with that number of members)
; at the present moment of time, plus a regression line
;

to plot-degree-distribution
  set-current-plot "Degree distribution"
  clear-plot  ;; erase what we plotted before
  set-plot-pen-color black
  set-plot-pen-mode 2 ;; plot points
  let max-degree max [length members] of networks 
;  let links sentence values-from firms [length partners] values-from networks [length members]
;  let max-degree max links
  let degree 1 ; only include nodes with at least one link
  let sumx 0 ;; for regression line
  let sumy 0
  let sumxy 0
  let sumxx 0
  let sumyy 0
  let n 0
  while [degree <= max-degree]
  [
    let matches networks with [length members = degree]
    if any? matches
;    let matches filter [? = degree ] links
;    if not empty? matches 
      [ let x log degree 10
        let y log (count matches) 10 
;       let y log (length matches) 10
        plotxy x y
        set sumx sumx + x
        set sumy sumy + y
        set sumxy sumxy + x * y
        set sumxx sumxx + x * x
        set sumyy sumyy + y * y
        set n n + 1
        ]
    set degree degree + 1
  ]
  if n > 1 [
   let slope  (n * sumxy - sumx * sumy) / (n * sumxx - sumx * sumx)
   let intercept  (sumy - slope * sumx) / n
   create-temporary-plot-pen "regression line"
   set-plot-pen-mode 0
   set-plot-pen-color red
   plot-pen-up
   plotxy 0 intercept
   plot-pen-down
   ifelse slope = 0 [
     plotxy intercept intercept  ;; regression line is parallel to x-axis
     ]
     [
     plotxy -1 * intercept / slope 0
     ]
   set-current-plot "Degree distn corr. coeff"
   let divisor (n * sumyy - sumy * sumy) * (n * sumxx - sumx * sumx)
   let r2 0
   if abs(divisor) > 0.00000001 [
     let numerator (n * sumxy - sumx * sumy)
     set r2 numerator * numerator / divisor
     ]
   plot r2
   set-current-plot "gamma"
   plot -1 * slope
  ]
end

; set the colour and size of the displayed icon according to whether it is the 
; firm of a network,
; a firm set up at the beginning of the run, or a start-up.  Adjust the icon size
; to show the firm's capital

to set-size-and-color
  ifelse product < raw-materials [ set color brown ]
    [ ifelse product > nProducts - end-products  [ set color blue  ]
      [ set color green ] ]
  ifelse network-firm [ set color color - 3 ]
    [ if age < 10 and step > 10 [ set color color + 3 ] ]
  if capital > 0 [ set size (log capital 10) * diam ]
  set heading 0
end
 
; arrange the icons on the display in a way that brings linked firms closer together

to layout-firms
  let delta 0
  
  
  without-interruption [
    no-display
    ask edges with [ kind = "supplier" ] 
      [ set hidden? not showSales ]
    ask edges with [ kind = "partner" ] 
      [ set hidden? not showPartners ]
    ask edges with [ kind = "networks" ] 
      [ set hidden? not showNetworks ]
    ask edges [
      if a = nobody or b = nobody [ die ]
      set delta (spring-force * (size - spring-length)) / 2.0
      if kind != "network" [ set delta delta - 2.0 ]
      ask a [set heading towards-nowrap [b] of myself jump-nowrap delta]
      ask b [set heading towards-nowrap [a] of myself jump-nowrap delta]
      ]
    ask firms [
      ask firms with [self != myself] [
        set delta distance-nowrap myself
        set delta mutual-repulsion / (delta * delta)
        set heading towards-nowrap myself
        jump-nowrap (- delta)
        ]
      ]
    ask edges [ adjust-edge ]
    ask firms [ set-size-and-color ]
    display
  ]
end

to arrange 
  let t 0
  
  display
  if mouse-down? [
    set t closest-xy mouse-xcor mouse-ycor firms ;
    while [mouse-down?] [
      ask t [setxy mouse-xcor mouse-ycor]
      no-display
      ask edges with [a = t or b = t][adjust-edge]
      display
    ]
  ]
  if plotting [ layout-firms ]
end

; create a turtle that will show as a line on the network diagram connecting two firms

to connect-edge [ other-node knd ]
  hatch 1 [
    set breed edges
    set a myself
    set b other-node
    set kind knd
    ifelse knd = "network" [ set color lime set hidden? not showNetworks ]
      [ ifelse knd = "partner" [ set color magenta set hidden? not showPartners ]
        [ set color sky set hidden? not showSales ] 
      ] 
    adjust-edge
  ]
end

; kill off all the lines betwen myself and any other agent, of the given kind

to disconnect-edges [ knd ]
  ask edges with [ a = myself or b = myself and kind = knd ] [ die ]
end

; move the line so that once again it links the two end nodes (firms)

to adjust-edge
  setxy [xcor] of a [ycor] of a
  set size distance-nowrap b
  set heading towards-nowrap b
  jump size / 2
end

; report the sign of the number

to-report sign [ num ]
  ifelse num < 0 [ report -1 ] [ report 1 ]
end

; report the closest agent to the point x,y

to-report closest-xy [x y agent-set]
  ;report min-one [ distancexy-nowrap x y ] of agent-set 
  report min-one-of agent-set [ distancexy-nowrap x y ] 
end

; jump a distance 'dist' along the current heading, but don't wrap around the
; display edges

to jump-nowrap [dist]
  let x 0
  let y 0
  

  set x xcor + dist * dx
  set y ycor + dist * dy
  if (abs x) > x-max [set x sign x * (x-max - (1 - friction) * ((abs x) - x-max))]
  if (abs y) > y-max [set y sign y * (y-max - (1 - friction) * ((abs y) - y-max))]
  setxy x y
end

; dump a file describing the current network(s) in a format that UCINET will understand
;

to write-network
  file-open user-new-file
  file-print (word "dl n="  count(firms)  ", format=nodelist1")
  file-print "labels embedded"
  file-print "data:"
  ask firms [
    file-write [who] of self
    without-interruption [
     foreach partners [
       if is-agent? ? [ file-write [who] of ? ]
       ]
     if is-agent? network [
       foreach [members] of network [
          file-write [who] of ?
          ]
       ]
     file-print ""
     ]
   ]
  file-close
end

; end
@#$#@#$#@
GRAPHICS-WINDOW
394
174
583
384
89
89
0.9441341
1
10
1
1
1
0
1
1
1
-89
89
-89
89
0
0
1
ticks

CC-WINDOW
5
897
981
992
Command Center
0

BUTTON
14
16
77
49
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
639
557
972
712
Population
Time
N
0.0
100.0
0.0
100.0
true
true
PENS
"Firms" 1.0 0 -16777216 true
"Networks" 1.0 0 -10899396 true

PLOT
609
713
793
850
Concentration
Time
Index * 100
0.0
100.0
0.0
1.0
true
false
PENS
"Herfindahl" 1.0 0 -14835848 true

PLOT
635
354
972
550
Collaboration
Time
%
0.0
100.0
0.0
100.0
true
true
PENS
"In partnership" 1.0 0 -5825686 true
"In network" 1.0 0 -10899396 true
"Connectedness" 1.0 0 -13840069 true

BUTTON
15
55
78
88
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

CHOOSER
12
211
185
256
partnership-strategy
partnership-strategy
"conservative" "progressive"
1

MONITOR
14
96
186
141
Activity
activity
3
1
11

PLOT
635
176
971
351
Transactions
Time
%
0.0
100.0
0.0
100.0
true
true
PENS
"Firms selling" 1.0 0 -7500403 true
"Sales" 1.0 0 -6459832 true

SLIDER
195
253
367
286
nInputs
nInputs
3
6
5
1
1
NIL
HORIZONTAL

SLIDER
195
214
367
247
nProducts
nProducts
0
100
30
1
1
NIL
HORIZONTAL

SWITCH
194
361
366
394
open-system
open-system
0
1
-1000

PLOT
214
400
425
551
Dynamics
Time
N
0.0
100.0
0.0
5.0
true
true
PENS
"Successes" 1.0 0 -13791810 true
"Start-ups" 1.0 0 -2674135 true

SWITCH
14
173
186
206
collaboration
collaboration
0
1
-1000

SLIDER
195
290
367
323
n-big-firms
n-big-firms
0
10
0
1
1
NIL
HORIZONTAL

PLOT
429
400
629
551
Networks
Size
Count
0.0
10.0
0.0
5.0
false
false
PENS
"default" 1.0 1 -10899396 true

SLIDER
195
174
367
207
nFirms
nFirms
0
200
100
5
1
NIL
HORIZONTAL

MONITOR
85
10
156
55
NIL
step
0
1
11

SLIDER
14
262
186
295
attractiveness-threshold
attractiveness-threshold
0
1
0.4
0.1
1
NIL
HORIZONTAL

TEXTBOX
397
98
562
175
Brown factory: raw material  \nGreen factory: firm   \nBlue factory: end-user\nBlue line: supplier-customer  \nRed line: partnership  \nGreen line: network
10
0.0
0

PLOT
12
400
211
550
Capital
Time
log 10 �
0.0
100.0
0.0
10.0
true
false
PENS
"Average" 1.0 0 -2674135 true
"+1 St Dev" 1.0 0 -955883 true
"-1 St Dev" 1.0 0 -6459832 true

SWITCH
391
62
494
95
showSales
showSales
1
1
-1000

SWITCH
496
26
619
59
showPartners
showPartners
1
1
-1000

SWITCH
496
62
621
95
showNetworks
showNetworks
1
1
-1000

PLOT
13
713
212
848
Degree distribution
Log (k)
Log (degree distribution)
0.0
2.0
0.0
2.0
true
false
PENS
"default" 1.0 0 -16777216 true

PLOT
215
713
406
848
Degree distn corr. coeff
Time
R2
0.0
10.0
0.0
1.0
true
false
PENS
"default" 1.0 0 -16777216 true

PLOT
408
713
608
848
gamma
Time
Gamma (slope of deg. distn)
0.0
10.0
0.0
1.0
true
false
PENS
"default" 1.0 0 -16777216 true

PLOT
796
713
972
850
Partners
Number
Frequency
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true

BUTTON
11
850
145
883
Write UCINET file
write-network
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
13
361
185
394
nHEIs
nHEIs
0
50
20
1
1
NIL
HORIZONTAL

SLIDER
13
323
187
356
HEI-success-threshold
HEI-success-threshold
0
50
30
1
1
%
HORIZONTAL

PLOT
13
557
339
711
knowledge-plot
NIL
NIL
0.0
10.0
0.0
0.05
true
true
PENS
"knowledge-flow" 1.0 0 -16777216 true
"1st Dev knowledge" 1.0 0 -2674135 true
"knowledge" 1.0 0 -955883 true
"rel-knowledge-flow" 1.0 0 -7500403 true

SWITCH
390
27
480
60
plotting
plotting
0
1
-1000

PLOT
343
557
639
712
knowledge-flow-plot
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true
"knowledge-flow" 1.0 0 -16777216 true

SLIDER
195
29
367
62
nRuns
nRuns
0
100
50
1
1
NIL
HORIZONTAL

SLIDER
195
68
367
101
current_run
current_run
0
100
0
1
1
NIL
HORIZONTAL

MONITOR
85
48
156
93
NIL
count firms with [ not HEI ]
17
1
11

SLIDER
195
105
367
138
maxSteps
maxSteps
0
5000
2000
100
1
NIL
HORIZONTAL

SLIDER
195
326
367
359
success-threshold
success-threshold
0
1000
200
20
1
NIL
HORIZONTAL

TEXTBOX
194
10
280
28
Multirun-setup
11
15.0
1

TEXTBOX
16
301
166
319
HEI-settings\n
11
14.0
1

TEXTBOX
16
151
166
169
Partnership settings\n
11
24.0
1

TEXTBOX
198
151
348
169
Experiment Setup
11
15.0
1

TEXTBOX
390
10
540
28
Realtime plot settings
11
24.0
1

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7500403 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7500403 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7500403 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7500403 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7500403 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7500403 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7500403 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7500403 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7500403 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -1184463 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7500403 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7500403 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7500403 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7500403 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7500403 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7500403 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7500403 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7500403 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7500403 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7500403 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7500403 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7500403 true 167 47 159 82
Line -7500403 true 136 47 145 81
Circle -7500403 true true 165 45 8
Circle -7500403 true true 134 45 6
Circle -7500403 true true 133 44 7
Circle -7500403 true true 133 43 8

circle
false
0
Circle -7500403 true true 35 35 230

directed line
true
0
Line -7500403 true 150 0 150 300
Circle -7500403 true true 135 16 31

factory
true
0
Polygon -7500403 true true 1 152 2 296 299 296 299 3 264 3 265 91 152 2 2 119

line
true
0
Line -7500403 true 150 0 150 300

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

person
false
0
Circle -7500403 true true 155 20 63
Rectangle -7500403 true true 158 79 217 164
Polygon -7500403 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7500403 true true 216 83 267 123 248 143 215 107
Polygon -7500403 true true 167 163 145 234 183 234 183 163
Polygon -7500403 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7500403 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7500403 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7500403 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8630108 true false 195 75 195 120 240 120 240 75
Polygon -8630108 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7500403 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8630108 true false 90 210 105 225 120 210
Polygon -8630108 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7500403 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8630108 true false 210 210 195 225 180 210
Polygon -8630108 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7500403 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7500403 true true 15 105 105 165
Rectangle -7500403 true true 45 90 105 105
Polygon -7500403 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7500403 true true 105 120 263 195
Rectangle -7500403 true true 108 195 259 201
Rectangle -7500403 true true 114 201 252 210
Rectangle -7500403 true true 120 210 243 214
Rectangle -7500403 true true 115 114 255 120
Rectangle -7500403 true true 128 108 248 114
Rectangle -7500403 true true 150 105 225 108
Rectangle -7500403 true true 132 214 155 270
Rectangle -7500403 true true 110 260 132 270
Rectangle -7500403 true true 210 214 232 270
Rectangle -7500403 true true 189 260 210 270
Line -7500403 true 263 127 281 155
Line -7500403 true 281 155 281 192

wolf-left
false
3
Polygon -6459832 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6459832 true true 87 80 79 55 76 79
Polygon -6459832 true true 81 75 70 58 73 82
Polygon -6459832 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6459832 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6459832 true true 116 140 114 189 105 137
Rectangle -6459832 true true 109 150 114 192
Rectangle -6459832 true true 111 143 116 191
Polygon -6459832 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6459832 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6459832 true true 214 134 203 168 192 148
Polygon -6459832 true true 204 151 203 176 193 148
Polygon -6459832 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6459832 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6459832 true true 201 99 214 69 215 99
Polygon -6459832 true true 207 98 223 71 220 101
Polygon -6459832 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6459832 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6459832 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6459832 true true 48 143 58 141
Polygon -6459832 true true 46 136 68 137
Polygon -6459832 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6459832 true true 38 138 66 149
Polygon -6459832 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6459832 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 4.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
